@@ -227,7 +227,7 @@ memberI f gs = memberGB f (gb gs)
 -- ie V(I+J) = V(I) intersect V(J)
 sumI :: (Fractional k, Ord k, Monomial m, Ord m, Algebra k m) =>
      [Vect k m] -> [Vect k m] -> [Vect k m]
-sumI fs gs = gb $ fs ++ gs
+sumI fs gs = gb (fs ++ gs)
 
 -- Cox et al, p183
 -- |Given ideals I and J, their product I.J is the ideal generated by all products {f.g | f \<- I, g \<- J}.
@@ -363,6 +363,13 @@ hilbertSeriesQA vs gs = hilbertSeriesQA' $ ltIdeal gs
           as <-> [] = as
           [] <-> bs = map negate bs
 
+-- |In the case where every variable v occurs in some generator g of the homogeneous ideal (the usual case),
+-- then the vs can be inferred from the gs.
+-- @hilbertSeriesQA' gs@ returns the Hilbert series for the quotient algebra k[vs]/\<gs\>.
+hilbertSeriesQA' :: (Fractional k, Ord k, MonomialConstructor m, Ord (m v), Monomial (m v), Algebra k (m v)) =>
+    [Vect k (m v)] -> [Integer]
+hilbertSeriesQA' gs = hilbertSeriesQA vs gs where vs = toSet (concatMap vars gs)
+
 -- |For i \>\> 0, the Hilbert function becomes a polynomial in i, called the Hilbert polynomial.
 hilbertPolyQA :: (Fractional k, Ord k, Monomial m, Ord m, Algebra k m) =>
      [Vect k m] -> [Vect k m] -> GlexPoly Q String
@@ -372,5 +379,11 @@ hilbertPolyQA vs gs = hilbertPolyQA' (ltIdeal gs) i
           hilbertPolyQA' [] x = product [ x + fromInteger j | j <- [1..n-1] ] / (fromInteger $ product [1..n-1])
           hilbertPolyQA' (m:ms) x = hilbertPolyQA' ms x - hilbertPolyQA' (ms `quotientP` m) (x - fromIntegral (deg m))
 
+hilbertPolyQA' :: (Fractional k, Ord k, MonomialConstructor m, Ord (m v), Monomial (m v), Algebra k (m v)) =>
+    [Vect k (m v)] -> GlexPoly Q String
+hilbertPolyQA' gs = hilbertPolyQA vs gs where vs = toSet (concatMap vars gs)
+
 -- The dimension of a variety
-dim vs gs = 1 + deg (hilbertPolyQA vs gs)
\ No newline at end of file
+dim vs gs = 1 + deg (hilbertPolyQA vs gs)
+
+dim' gs = 1 + deg (hilbertPolyQA' gs)
\ No newline at end of file
