@@ -1,6 +1,6 @@
 -- Copyright (c) 2011, David Amos. All rights reserved.
 
-{-# LANGUAGE GeneralizedNewtypeDeriving, MultiParamTypeClasses, FlexibleInstances #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving, MultiParamTypeClasses, FlexibleInstances, DeriveFunctor #-}
 
 -- |A module defining the algebra of commutative polynomials over a field k.
 -- Polynomials are represented as the free k-vector space with the monomials as basis.
@@ -14,6 +14,7 @@
 module Math.CommutativeAlgebra.Polynomial where
 
 import Math.Core.Field
+import Math.Core.Utils (toSet)
 import Math.Algebras.VectorSpace
 import Math.Algebras.TensorProduct
 import Math.Algebras.Structures
@@ -22,7 +23,6 @@ import Math.Algebras.Structures
 -- Multiplication is defined by the Mon (monoid) class. Division is defined in this class.
 -- The functions here are primarily intended for internal use only.
 class (Eq m, Show m, Mon m) => Monomial m where
-    -- mdivmaybe :: m -> m -> Maybe m
     mdivides :: m -> m -> Bool
     mdiv :: m -> m -> m
     mgcd :: m -> m -> m
@@ -32,8 +32,6 @@ class (Eq m, Show m, Mon m) => Monomial m where
 
 -- mlcm m1 m2 = let m = mgcd m1 m2 in mmult m1 (mdiv m2 m)
 
--- mdividesne m1 m2 = m1 /= m2 && mdivides m1 m2
-
 mproperlydivides m1 m2 = m1 /= m2 && mdivides m1 m2
 
 
@@ -65,7 +63,7 @@ var = return . mvar
 --
 -- No Ord instance is defined for MonImpl v, so it cannot be used as the basis for a free vector space of polynomials.
 -- Instead, several different newtype wrappers are provided, corresponding to different monomial orderings.
-data MonImpl v = M Int [(v,Int)] deriving (Eq)
+data MonImpl v = M Int [(v,Int)] deriving (Eq, Functor)
 -- The initial Int is the degree of the monomial. Storing it speeds up equality tests and comparisons
 
 instance Show v => Show (MonImpl v) where
@@ -78,13 +76,6 @@ instance (Ord v) => Mon (MonImpl v) where
     mmult (M si xis) (M sj yjs) = M (si+sj) $ addmerge xis yjs
 
 instance (Ord v, Show v) => Monomial (MonImpl v) where
-{-
-    mdivmaybe m1 m2 =
-        let m@(M s xis) = mdiv m1 m2
-        in if s < 0 || any (< 0) (map snd xis)
-           then Nothing
-           else Just m
--}
     mdivides (M si xis) (M sj yjs) = si <= sj && mdivides' xis yjs where
         mdivides' ((x,i):xis) ((y,j):yjs) =
             case compare x y of
@@ -131,7 +122,7 @@ instance MonomialConstructor MonImpl where
 --
 -- Lex stands for lexicographic ordering.
 -- For example, in Lex ordering, monomials up to degree two would be ordered as follows: x^2+xy+xz+x+y^2+yz+y+z^2+z+1.
-newtype Lex v = Lex (MonImpl v) deriving (Eq, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
+newtype Lex v = Lex (MonImpl v) deriving (Eq, Functor, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
 
 instance Show v => Show (Lex v) where
     show (Lex m) = show m
@@ -177,7 +168,7 @@ instance (Num k, Ord v, Show v) => Algebra k (Lex v) where
 --
 -- Glex stands for graded lexicographic. Thus monomials are ordered first by degree, then by lexicographic order.
 -- For example, in Glex ordering, monomials up to degree two would be ordered as follows: x^2+xy+xz+y^2+yz+z^2+x+y+z+1.
-newtype Glex v = Glex (MonImpl v) deriving (Eq, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
+newtype Glex v = Glex (MonImpl v) deriving (Eq, Functor, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
 
 instance Show v => Show (Glex v) where
     show (Glex m) = show m
@@ -213,7 +204,7 @@ instance (Num k, Ord v, Show v) => Algebra k (Glex v) where
 -- For example, in Grevlex ordering, monomials up to degree two would be ordered as follows: x^2+xy+y^2+xz+yz+z^2+x+y+z+1.
 --
 -- In general, Grevlex leads to the smallest Groebner bases.
-newtype Grevlex v = Grevlex (MonImpl v) deriving (Eq, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
+newtype Grevlex v = Grevlex (MonImpl v) deriving (Eq, Functor, Mon, Monomial, MonomialConstructor) -- GeneralizedNewtypeDeriving
 
 instance Show v => Show (Grevlex v) where
     show (Grevlex m) = show m
@@ -243,7 +234,7 @@ instance (Num k, Ord v, Show v) => Algebra k (Grevlex v) where
 
 -- ELIMINATION ORDER
 
-data Elim2 a b = Elim2 !a !b deriving (Eq)
+data Elim2 a b = Elim2 !a !b deriving (Eq, Functor)
 
 instance (Ord a, Ord b) => Ord (Elim2 a b) where
     compare (Elim2 a1 b1) (Elim2 a2 b2) = compare (a1,b1) (a2,b2)
@@ -260,7 +251,6 @@ instance (Mon a, Mon b) => Mon (Elim2 a b) where
     mmult (Elim2 a1 b1) (Elim2 a2 b2) = Elim2 (mmult a1 a2) (mmult b1 b2)
 
 instance (Monomial a, Monomial b) => Monomial (Elim2 a b) where
-    -- mdivmaybe :: Ord v => m v -> m v -> Maybe (m v)
     mdivides (Elim2 a1 b1) (Elim2 a2 b2) = mdivides a1 a2 && mdivides b1 b2
     mdiv (Elim2 a1 b1) (Elim2 a2 b2) = Elim2 (mdiv a1 a2) (mdiv b1 b2)
     mgcd (Elim2 a1 b1) (Elim2 a2 b2) = Elim2 (mgcd a1 a2) (mgcd b1 b2)
@@ -325,6 +315,11 @@ subst f vs = f `bind` sub
                   Nothing -> error ("eval: no binding given for " ++ show x)
 -- The type could be more general than this, but haven't so far found a use case
 
+-- |List the variables used in a polynomial
+vars :: (Num k, Ord k, MonomialConstructor m, Ord (m v)) =>
+     Vect k (m v) -> [Vect k (m v)]
+vars f = toSet [ var v | (m,_) <- terms f, v <- map fst (mindices m) ]
+
 
 -- DIVISION ALGORITHM FOR POLYNOMIALS
 
@@ -377,23 +372,19 @@ quotRemMP f gs = quotRemMP' f (replicate n 0, 0) where
         in quotRemMP' h' (reverse us', r+lth)
     splitlt (V (t:ts)) = (V [t], V ts)
 
-{-
--- version using mdivmaybe - surprisingly, this seems to be slower
-quotRemMP2 f gs = quotRemMP' f (replicate n 0, 0) where
-    n = length gs
-    quotRemMP' 0 (us,r) = (us,r)
-    quotRemMP' h (us,r) = divisionStep h (gs,[],us,r)
-    divisionStep h (g:gs,us',u:us,r) =
-        case tdivmaybe (lt h) (lt g) of
-        Nothing -> divisionStep h (gs,u:us',us,r)
-        Just t -> let h' = h - V [t] * g
-                      u' = u + V [t]
-                  in quotRemMP' h' (reverse us' ++ u':us, r)
-    divisionStep h ([],us',[],r) =
-        let (lth,h') = splitlt h
-        in quotRemMP' h' (reverse us', r+lth)
-    splitlt (V (t:ts)) = (V [t], V ts)
--}
+
+rewrite f gs = rewrite' (f,0) gs where
+    rewrite' (0,r) _ = r
+    rewrite' (l,r) (h:hs) =
+        if lt h `tdivides` lt l -- if lhs of "rewrite rule" h matches
+        then let l' = l - V [lt l `tdiv` lt h] * h -- apply rewrite rule to eliminate leading term
+             in rewrite' (l',r) gs -- then start again and try to eliminate the new lt.
+        else rewrite' (l,r) hs -- else try the next potential divisor
+    rewrite' (l,r) [] = -- none of the rewrite rules matches lt l
+        let (h,t) = split l
+        in rewrite' (t, r + h) gs -- so move it into the remainder r, and try to rewrite the other terms
+    split (V (t:ts)) = (V [t], V ts)
+
 
 infixl 7 %%
 
@@ -403,7 +394,8 @@ infixl 7 %%
 -- and is zero if and only if f is in I.
 (%%) :: (Fractional k, Monomial m, Ord m, Algebra k m) =>
      Vect k m -> [Vect k m] -> Vect k m
-f %% gs = r where (_,r) = quotRemMP f gs
+f %% gs = rewrite f gs
+-- f %% gs = r where (_,r) = quotRemMP f gs
 
 
 -- |As a convenience, a partial instance of Fractional is defined for polynomials.
