@@ -15,7 +15,9 @@ import Math.Test.TCombinatorics.TDigraph
 import Math.Test.TCombinatorics.TIncidenceAlgebra
 import Math.Test.TCombinatorics.TMatroid
 import Math.Test.TCombinatorics.TPoset
+import Math.Test.TCommutativeAlgebra.TPolynomial
 import Math.Test.TCommutativeAlgebra.TGroebnerBasis
+import Math.Test.TNumberTheory.TPrimeFactor
 import Math.Test.TProjects.TMiniquaternionGeometry
 
 
@@ -46,5 +48,7 @@ hunitAll = runTestTT $ TestList [
     testlistIncidenceAlgebra,
     testlistMatroid,
     testlistPoset,
-    testlistGroebnerBasis
-    ]
\ No newline at end of file
+    testlistPolynomial,
+    testlistGroebnerBasis,
+    testlistPrimeFactor
+    ]
